==> lowcode-render-1.1.3.tgz: package/src/utils/query.ts <==
import {
  IDataSourceQuery,
  deserializeDynamicMap,
  EComponentType,
  normalizeQuery,
  generateHandlerParamsDynamicMap,
  EQueryType,
} from '@cloudbase/cals';
import { IReactionDisposer, autorun, observable, toJS } from 'mobx';
import {
  IGenerateContext,
  IGenerateOptions,
  generateDataBinds,
  generateListnerInstances,
  splitDynamicMap,
} from './adapter/generate';
import { generateEmit, mergeDynamic2StaticData } from './runtime';
import { debounce, get } from 'lodash-es';

export interface IQueryContext extends IGenerateContext {
  $w: any;
}

const enum EQUERY_PARAM_TYPE {
  ARRAY = 'ARRAY',
  BOOLEAN = 'BOOLEAN',
  NUMBER = 'NUMBER',
  OBJECT = 'OBJECT',
  STRING = 'STRING',
}

const enum QUERY_RESOURCE {
  SQL = 'mysql-plugin',
  SQLSERVER = 'mssql-plugin',
  CLOUDCONNECTOR = 'apis-plugin',
}

const enum QUERY_PATH {
  SQL = '/WeDa/Query/V1/RunPluginQuery',
  SQLSERVER = '/WeDa/Query/V1/RunPluginQuery',
  CLOUDCONNECTOR = '/WeDa/Connector/v1/RunApisQuery',
}

const KEYS = ['id', 'label', 'description', 'data', 'error', 'isFetching', 'trigger', 'reset'];

export class Query {
  #schema: IDataSourceQuery;
  #context: IQueryContext;
  #triggered = false;
  initPromise: Promise<void>;
  #initedResolver = (value) => {};
  #disposes: IReactionDisposer[] = [];
  #dataBinds: Record<string, Function> = {};
  #action: (data?, options?, additionalScope?) => Promise<{ profile?: any; value: any }>;
  #paramsRef: { current: any } = observable({ current: null });
  #emit: (eventName, data) => Promise<any>;
  #currentRequestKey = null;
  #observableValue: { data: any; error: Error; isFetching: boolean; profile: any } = observable({
    data: null,
    error: null,
    isFetching: false,
    profile: null,
  });
  #runQueryMap: Record<
    EQueryType.Sql | EQueryType.Sqlserver | EQueryType.CloudConnector,
    {
      QueryResource: string;
      QueryPath: string;
    }
  > = {
    sql: { QueryResource: QUERY_RESOURCE.SQL, QueryPath: QUERY_PATH.SQL },
    sqlserver: { QueryResource: QUERY_RESOURCE.SQLSERVER, QueryPath: QUERY_PATH.SQLSERVER },
    'cloud-connector': { QueryResource: QUERY_RESOURCE.CLOUDCONNECTOR, QueryPath: QUERY_PATH.CLOUDCONNECTOR },
  };

  constructor({
    schema,
    context,
    options,
  }: {
    schema: IDataSourceQuery;
    context: IQueryContext;
    options?: IGenerateOptions;
  }) {
    const { looseError = false, useScopeWidgetData = false } = options;
    const { $w, $app, $page } = context;
    this.#schema = normalizeQuery(schema);
    this.#context = context;

    this.initPromise = new Promise((resolve) => {
      this.#initedResolver = resolve;
    });

    if (this.#schema?.trigger === 'auto') {
      this.#disposes.push(
        autorun(
          (r) => {
            // r.trace(true);
            try {
              const data = this.#resolveParams({}, { combainErrors: true });
              if (this.#schema?.type === EQueryType.GeneralFunc) {
                // 因为要收集依赖，因此不能 debounce
                this.#innerTrigger(data);
              } else if (this.#triggered) {
                this.#debounceTrigger(data);
              }
            } catch (e) {
              console.error(e);
            } finally {
              this.#initedResolver(true);
            }
          },
          { delay: 50 },
        ),
      );
    } else {
      this.#initedResolver(true);
    }

    const deserializeContext = $page?.id
      ? {
          page: { id: $page.id, type: EComponentType.PAGE },
        }
      : {};

    const processedParams = deserializeDynamicMap(
      deserializeContext,
      generateHandlerParamsDynamicMap(this.#schema.handler),
    );

    const { staticData, dataBinds = [] } = splitDynamicMap(processedParams);

    this.#paramsRef.current = staticData;
    this.#dataBinds = generateDataBinds(
      { $page, $app },
      dataBinds?.filter((item) => {
        return !/[\s+\-*/!&|%]*?SERVER\./.test(item.propertyPath);
      }),
      {
        looseError,
        useScopeWidgetData,
        ...options,
        key: `${this.id} method.params`,
      },
    );

    const generalFunQuery = async function (data, options: { debug?: boolean } = {}, additionalScope) {
      let { debug = false } = options;
      let start = Date.now();
      const innerFunction = new window.Function(
        '__data__,__params__,$w',
        `return (\n${this.#schema.handler?.[':code'] || '()=>{}'}\n)({params:__params__,data:__data__})`,
      );

      let result = await innerFunction(data, additionalScope, $w);

      const profile = {
        requestTimeCost: Date.now() - start,
      };

      return {
        profile: debug ? profile : undefined,
        value: result,
      };
    };

    const runQueryType = async function (data, options: { debug?: boolean } = {}) {
      let { debug = false } = options;

      const requestData = {
        EnvId: $w.env.envId,
        Name: data.sqlTemplateId,
        Parameter: JSON.stringify(
          Object.entries(data.params || {}).reduce((list, [key, value]) => {
            if (value !== undefined) {
              let type = EQUERY_PARAM_TYPE.OBJECT;
              const typeofValue = typeof value;
              switch (typeofValue) {
                case 'boolean': {
                  type = EQUERY_PARAM_TYPE.BOOLEAN;
                  break;
                }
                case 'number': {
                  type = EQUERY_PARAM_TYPE.NUMBER;
                  break;
                }
                case 'string': {
                  type = EQUERY_PARAM_TYPE.STRING;
                  break;
                }
                default: {
                  if (Array.isArray(value)) {
                    type = EQUERY_PARAM_TYPE.ARRAY;
                  } else {
                    type = EQUERY_PARAM_TYPE.OBJECT;
                  }
                }
              }

              list.push({
                Key: key,
                Type: type,
                Value: type === EQUERY_PARAM_TYPE.STRING ? value : JSON.stringify(value),
              });
            }

            return list;
          }, []) || [],
        ),
      };

      let start = Date.now();
      const Data = await $w.cloud.callWedaApi({
        action: 'RunQuery',
        CallQuery: true,
        ...(this.#runQueryMap[this.#schema.type] || this.#runQueryMap[EQueryType.Sql]),
        EnvId: requestData.EnvId,
        QueryName: requestData.Name,
        data: requestData,
        BackendLatency: !!debug,
      });

      // const Data = await $w.cloud.callWedaApi({
      //   action: 'RunPluginQuery',
      //   data: { MainAppId: $w.app.id, ...requestData },
      // });

      const { ExecuteResultList = [], Total, BackendExecute, BackendLatency, Payload } = Data || {};
      const content = ExecuteResultList.join('');

      const profile = {
        backendExecude: Number(BackendExecute || 0),
        backendLatency: Number(BackendLatency || 0),
        /**
         * 中文时有问题
         * 但是不太关键，别用于关键逻辑判断
         */
        contentLength: content.length,
        requestTimeCost: Date.now() - start,
      };

      try {
        const encoder = new TextEncoder();
        const bytes = encoder.encode(content);
        profile.contentLength = bytes.length;
      } catch (e) {}

      let payload = Payload;
      try {
        if (Payload) {
          payload = JSON.parse(Payload);
        }
      } catch (e) {}

      return {
        profile: debug ? profile : undefined,
        value:
          this.#schema.type === EQueryType.CloudConnector
            ? payload
            : {
                records: ExecuteResultList.map((item) => JSON.parse(item)),
                total: Total || ExecuteResultList.length,
              },
      };
    };

    this.#action = async function (data, options, additionalScope) {
      const errorHandler = function (data) {
        throw new Error('Invalue query handler');
      };

      return this.#isRunQueryType
        ? runQueryType.call(this, data, options)
        : this.#schema.type === EQueryType.GeneralFunc
        ? generalFunQuery.call(this, data, options, additionalScope)
        : { value: await (get($w.app, this.#schema.handler?.name).call(this, data) || errorHandler(data)) };
    };

    this.#emit = async (trigger, eventData, originalEvent?) =>
      generateEmit($w)(
        trigger,
        generateListnerInstances(
          {
            $app,
            $page,
            actionsMap: (this.#context as any).actionsMap,
          },
          this.#schema.listeners || [],
          options,
        ),
        eventData,
        {},
        originalEvent,
        {},
        {},
        /**
         * 当 app 全局情况下
         * 不进行 pageId 判断
         */
        $w.page?.id,
        true,
      );

    return this;
  }

  get id(): string {
    return this.#schema?.id || '';
  }

  get label(): string {
    return this.#schema?.label || '';
  }

  get description(): string {
    return this.#schema?.description || '';
  }

  get data() {
    return this.#observableValue.data;
  }

  get error() {
    return this.#observableValue.error;
  }

  get isFetching() {
    return this.#observableValue.isFetching;
  }

  async trigger(additionalScope?, options?: { debug?: boolean }) {
    this.#triggered = true;
    let start = Date.now();
    const resolvedParams = this.#resolveParams(additionalScope);
    const resolveParamsTimecost = Date.now() - start;
    const res = await this.#innerTrigger(resolvedParams, options, additionalScope);
    if (res?.profile) {
      res.profile = {
        ...res.profile,
        trrigerTimeCost: Date.now() - start,
        resolveParamsTimecost,
      };
    }
    return res;
  }

  reset() {
    this.#observableValue.data = null;
    this.#observableValue.error = null;
    this.#observableValue.isFetching = false;
  }

  destroy() {
    this.#disposes.forEach((dispose) => dispose());
  }

  /**
   * 用于调试
   */
  _toSchema() {
    console.warn('调试使用，结构可能发生更改，请不要用于生产环境');
    return this.#schema;
  }

  async #innerTrigger(args, options?: { debug?: boolean }, additionalScope?: any) {
    this.#currentRequestKey = Date.now();
    const key = this.#currentRequestKey;
    try {
      this.#observableValue.isFetching = true;
      const actionArgs = [args?.[0]];
      actionArgs.push(options, additionalScope);

      const { profile, value: res } = await this.#action(...actionArgs);

      if (key === this.#currentRequestKey) {
        this.#observableValue.isFetching = false;
        this.#observableValue.data = res;
        this.#observableValue.error = null;
        this.#emit(`success`, res);
      }

      return profile ? { ...(res instanceof Object ? res : { value: res }), profile } : res;
    } catch (e) {
      if (key === this.#currentRequestKey) {
        this.#observableValue.isFetching = false;
        this.#observableValue.data = null;
        this.#observableValue.error = e;
        this.#emit(`fail`, e);
      }
      throw e;
    }
  }

  #debounceTrigger = debounce(this.#innerTrigger, 300);

  #resolveParams(additionalScope = {}, options?: { combainErrors: boolean }): any[] {
    /**
     * 这里万一其中某个字段计算失败
     * 好像会阻塞其他字段的计算
     * 从而导致 autorun 没有添加依赖
     */
    let { params = [] } =
      mergeDynamic2StaticData(
        toJS(this.#paramsRef.current),
        this.#dataBinds,
        {
          $w: this.#context.$w,
          paramsContext: additionalScope,
        },
        options?.combainErrors,
      ) || {};

    return params;
  }

  get #isRunQueryType() {
    return !!this.#runQueryMap[this.#schema.type];
  }

  toJSON() {
    return KEYS.reduce((obj, key) => {
      obj[key] = this[key];
      return obj;
    }, {});
  }
}

==> lowcode-render-1.1.7.tgz: package/src/utils/query.ts <==
import {
  IDataSourceQuery,
  deserializeDynamicMap,
  EComponentType,
  normalizeQuery,
  generateHandlerParamsDynamicMap,
  EQueryType,
} from '@cloudbase/cals';
import { IReactionDisposer, autorun, observable, toJS } from 'mobx';
import {
  IGenerateContext,
  IGenerateOptions,
  generateDataBinds,
  generateListnerInstances,
  splitDynamicMap,
} from './adapter/generate';
import { generateEmit, mergeDynamic2StaticData } from './runtime';
import { debounce, get } from 'lodash-es';

export interface IQueryContext extends IGenerateContext {
  $w: any;
}

const enum EQUERY_PARAM_TYPE {
  ARRAY = 'ARRAY',
  BOOLEAN = 'BOOLEAN',
  NUMBER = 'NUMBER',
  OBJECT = 'OBJECT',
  STRING = 'STRING',
}

const enum QUERY_RESOURCE {
  SQL = 'mysql-plugin',
  SQLSERVER = 'mssql-plugin',
  CLOUDCONNECTOR = 'apis-plugin',
}

const enum QUERY_PATH {
  SQL = '/WeDa/Query/V1/RunPluginQuery',
  SQLSERVER = '/WeDa/Query/V1/RunPluginQuery',
  CLOUDCONNECTOR = '/WeDa/Connector/v1/RunApisQuery',
}

const KEYS = ['id', 'label', 'description', 'data', 'error', 'isFetching', 'trigger', 'reset'];

export class Query {
  #schema: IDataSourceQuery;
  #context: IQueryContext;
  #triggered = false;
  initPromise: Promise<void>;
  #initedResolver = (value) => {};
  #disposes: IReactionDisposer[] = [];
  #dataBinds: Record<string, Function> = {};
  #action: (data?, options?, additionalScope?) => Promise<{ profile?: any; value: any }>;
  #paramsRef: { current: any } = observable({ current: null });
  #emit: (eventName, data) => Promise<any>;
  #currentRequestKey = null;
  #observableValue: { data: any; error: Error; isFetching: boolean; profile: any } = observable({
    data: null,
    error: null,
    isFetching: false,
    profile: null,
  });
  #runQueryMap: Record<
    EQueryType.Sql | EQueryType.Sqlserver | EQueryType.CloudConnector,
    {
      QueryResource: string;
      QueryPath: string;
    }
  > = {
    sql: { QueryResource: QUERY_RESOURCE.SQL, QueryPath: QUERY_PATH.SQL },
    sqlserver: { QueryResource: QUERY_RESOURCE.SQLSERVER, QueryPath: QUERY_PATH.SQLSERVER },
    'cloud-connector': { QueryResource: QUERY_RESOURCE.CLOUDCONNECTOR, QueryPath: QUERY_PATH.CLOUDCONNECTOR },
  };

  constructor({
    schema,
    context,
    options,
  }: {
    schema: IDataSourceQuery;
    context: IQueryContext;
    options?: IGenerateOptions;
  }) {
    const { looseError = false, useScopeWidgetData = false } = options;
    const { $w, $app, $page } = context;
    this.#schema = normalizeQuery(schema);
    this.#context = context;

    this.initPromise = new Promise((resolve) => {
      this.#initedResolver = resolve;
    });

    if (this.#schema?.trigger === 'auto') {
      this.#disposes.push(
        autorun(
          (r) => {
            // r.trace(true);
            try {
              const data = this.#resolveParams({}, { combainErrors: true });
              if (this.#schema?.type === EQueryType.GeneralFunc) {
                // 因为要收集依赖，因此不能 debounce
                this.#innerTrigger(data);
              } else if (this.#triggered) {
                this.#debounceTrigger(data);
              }
            } catch (e) {
              console.error(e);
            } finally {
              this.#initedResolver(true);
            }
          },
          { delay: 50 },
        ),
      );
    } else {
      this.#initedResolver(true);
    }

    const deserializeContext = $page?.id
      ? {
          page: { id: $page.id, type: EComponentType.PAGE },
        }
      : {};

    const processedParams = deserializeDynamicMap(
      deserializeContext,
      generateHandlerParamsDynamicMap(this.#schema.handler),
    );

    const { staticData, dataBinds = [] } = splitDynamicMap(processedParams);

    this.#paramsRef.current = staticData;
    this.#dataBinds = generateDataBinds(
      { $page, $app },
      dataBinds?.filter((item) => {
        return !/[\s+\-*/!&|%]*?SERVER\./.test(item.propertyPath);
      }),
      {
        looseError,
        useScopeWidgetData,
        ...options,
        key: `${this.id} method.params`,
      },
    );

    const generalFunQuery = async function (data, options: { debug?: boolean } = {}, additionalScope) {
      let { debug = false } = options;
      let start = Date.now();
      const innerFunction = new window.Function(
        '__data__,__params__,$w',
        `return (\n${
          this.#schema.handler?.[':code']?.replace?.(/;\s*$/, '') || '()=>{}'
        }\n)({params:__params__,data:__data__})`,
      );

      let result = await innerFunction(data, additionalScope, $w);

      const profile = {
        requestTimeCost: Date.now() - start,
      };

      return {
        profile: debug ? profile : undefined,
        value: result,
      };
    };

    const runQueryType = async function (data, options: { debug?: boolean } = {}) {
      let { debug = false } = options;

      const requestData = {
        EnvId: $w.env.envId,
        Name: data.sqlTemplateId,
        Parameter: JSON.stringify(
          Object.entries(data.params || {}).reduce((list, [key, value]) => {
            if (value !== undefined) {
              let type = EQUERY_PARAM_TYPE.OBJECT;
              const typeofValue = typeof value;
              switch (typeofValue) {
                case 'boolean': {
                  type = EQUERY_PARAM_TYPE.BOOLEAN;
                  break;
                }
                case 'number': {
                  type = EQUERY_PARAM_TYPE.NUMBER;
                  break;
                }
                case 'string': {
                  type = EQUERY_PARAM_TYPE.STRING;
                  break;
                }
                default: {
                  if (Array.isArray(value)) {
                    type = EQUERY_PARAM_TYPE.ARRAY;
                  } else {
                    type = EQUERY_PARAM_TYPE.OBJECT;
                  }
                }
              }

              list.push({
                Key: key,
                Type: type,
                Value: type === EQUERY_PARAM_TYPE.STRING ? value : JSON.stringify(value),
              });
            }

            return list;
          }, []) || [],
        ),
      };

      let start = Date.now();
      const Data = await $w.cloud.callWedaApi({
        action: 'RunQuery',
        CallQuery: true,
        ...(this.#runQueryMap[this.#schema.type] || this.#runQueryMap[EQueryType.Sql]),
        EnvId: requestData.EnvId,
        QueryName: requestData.Name,
        data: requestData,
        BackendLatency: !!debug,
      });

      // const Data = await $w.cloud.callWedaApi({
      //   action: 'RunPluginQuery',
      //   data: { MainAppId: $w.app.id, ...requestData },
      // });

      const { ExecuteResultList = [], Total, BackendExecute, BackendLatency, Payload } = Data || {};
      const content = ExecuteResultList.join('');

      const profile = {
        backendExecude: Number(BackendExecute || 0),
        backendLatency: Number(BackendLatency || 0),
        /**
         * 中文时有问题
         * 但是不太关键，别用于关键逻辑判断
         */
        contentLength: content.length,
        requestTimeCost: Date.now() - start,
      };

      try {
        const encoder = new TextEncoder();
        const bytes = encoder.encode(content);
        profile.contentLength = bytes.length;
      } catch (e) {}

      let payload = Payload;
      try {
        if (Payload) {
          payload = JSON.parse(Payload);
        }
      } catch (e) {}

      return {
        profile: debug ? profile : undefined,
        value:
          this.#schema.type === EQueryType.CloudConnector
            ? payload
            : {
                records: ExecuteResultList.map((item) => JSON.parse(item)),
                total: Total || ExecuteResultList.length,
              },
      };
    };

    this.#action = async function (data, options, additionalScope) {
      const errorHandler = function (data) {
        throw new Error('Invalue query handler');
      };

      return this.#isRunQueryType
        ? runQueryType.call(this, data, options)
        : this.#schema.type === EQueryType.GeneralFunc
        ? generalFunQuery.call(this, data, options, additionalScope)
        : { value: await (get($w.app, this.#schema.handler?.name).call(this, data) || errorHandler(data)) };
    };

    this.#emit = async (trigger, eventData, originalEvent?) =>
      generateEmit($w)(
        trigger,
        generateListnerInstances(
          {
            $app,
            $page,
            actionsMap: (this.#context as any).actionsMap,
          },
          this.#schema.listeners || [],
          options,
        ),
        eventData,
        {},
        originalEvent,
        {},
        {},
        /**
         * 当 app 全局情况下
         * 不进行 pageId 判断
         */
        $w.page?.id,
        true,
      );

    return this;
  }

  get id(): string {
    return this.#schema?.id || '';
  }

  get label(): string {
    return this.#schema?.label || '';
  }

  get description(): string {
    return this.#schema?.description || '';
  }

  get data() {
    return this.#observableValue.data;
  }

  get error() {
    return this.#observableValue.error;
  }

  get isFetching() {
    return this.#observableValue.isFetching;
  }

  async trigger(additionalScope?, options?: { debug?: boolean }) {
    this.#triggered = true;
    let start = Date.now();
    const resolvedParams = this.#resolveParams(additionalScope);
    const resolveParamsTimecost = Date.now() - start;
    const res = await this.#innerTrigger(resolvedParams, options, additionalScope);
    if (res?.profile) {
      res.profile = {
        ...res.profile,
        trrigerTimeCost: Date.now() - start,
        resolveParamsTimecost,
      };
    }
    return res;
  }

  reset() {
    this.#observableValue.data = null;
    this.#observableValue.error = null;
    this.#observableValue.isFetching = false;
  }

  destroy() {
    this.#disposes.forEach((dispose) => dispose());
  }

  /**
   * 用于调试
   */
  _toSchema() {
    console.warn('调试使用，结构可能发生更改，请不要用于生产环境');
    return this.#schema;
  }

  async #innerTrigger(args, options?: { debug?: boolean }, additionalScope?: any) {
    this.#currentRequestKey = Date.now();
    const key = this.#currentRequestKey;
    try {
      this.#observableValue.isFetching = true;
      const actionArgs = [args?.[0]];
      actionArgs.push(options, additionalScope);

      const { profile, value: res } = await this.#action(...actionArgs);

      if (key === this.#currentRequestKey) {
        this.#observableValue.isFetching = false;
        this.#observableValue.data = res;
        this.#observableValue.error = null;
        this.#emit(`success`, res);
      }

      return profile ? { ...(res instanceof Object ? res : { value: res }), profile } : res;
    } catch (e) {
      if (key === this.#currentRequestKey) {
        this.#observableValue.isFetching = false;
        this.#observableValue.data = null;
        this.#observableValue.error = e;
        this.#emit(`fail`, e);
      }
      throw e;
    }
  }

  #debounceTrigger = debounce(this.#innerTrigger, 300);

  #resolveParams(additionalScope = {}, options?: { combainErrors: boolean }): any[] {
    /**
     * 这里万一其中某个字段计算失败
     * 好像会阻塞其他字段的计算
     * 从而导致 autorun 没有添加依赖
     */
    let { params = [] } =
      mergeDynamic2StaticData(
        toJS(this.#paramsRef.current),
        this.#dataBinds,
        {
          $w: this.#context.$w,
          paramsContext: additionalScope,
        },
        options?.combainErrors,
      ) || {};

    return params;
  }

  get #isRunQueryType() {
    return !!this.#runQueryMap[this.#schema.type];
  }

  toJSON() {
    return KEYS.reduce((obj, key) => {
      obj[key] = this[key];
      return obj;
    }, {});
  }
}
